package at.ac.uibk.gitsearch.service;

import static at.ac.uibk.gitsearch.web.rest.AccountResourceIT.TEST_USER_LOGIN;
import static org.hamcrest.Matchers.anyOf;
import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.everyItem;
import static org.hamcrest.Matchers.hasItemInArray;
import static org.hamcrest.Matchers.hasProperty;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.node.NodeValidationException;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.security.test.context.support.WithMockUser;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestServerConfiguration;
import at.ac.uibk.gitsearch.service.dto.SearchInputDTO;
import at.ac.uibk.gitsearch.service.dto.SearchInputMetadataDTO;
import at.ac.uibk.gitsearch.service.dto.SearchResultsDTO;

/**
 * Integration tests for {@link MailService}.
 */
@SpringBootTest(classes = GitsearchApp.class)
//@WithMockUser(value = TEST_USER_LOGIN)
public class SearchServiceIT {
	
	public static final Logger LOGGER = LogManager.getLogger(SearchServiceIT.class);

	@Autowired
	private SearchService searchService;

	@BeforeAll
	public static void setUpESServer() throws IOException, NodeValidationException {
		LOGGER.info("Starting es test server");
		ElasticSearchTestServerConfiguration.getTestInstance().startTestNode();
		LOGGER.info("Started  es test server");
	}

	@BeforeEach
	public void setup() {
	//	MockitoAnnotations.initMocks(this);
		LOGGER.info("in setup");
	}

	@Test
	public void testFullTextSearch() throws Exception {
		LOGGER.info("starting testFullTextSearch");
		final String WIEN = "Wien";
		final SearchInputMetadataDTO searchMetadata = new SearchInputMetadataDTO(null, null, null, null, null);
		SearchInputDTO searchQuery = new SearchInputDTO(WIEN, searchMetadata, null, null, null, 0);
		SearchResultsDTO searchResultPage = searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

		org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
		org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
		org.junit.Assert.assertThat(searchResultPage.getSearchResult(),
				everyItem(anyOf(hasProperty("metadata", hasProperty("description", containsString(WIEN))),
						hasProperty("metadata", hasProperty("title", containsString(WIEN)))
				// hasProperty("keywords", containsString("Wien"))
				)));

		LOGGER.info("exiting testFullTextSearch");
	}

	@Test
	public void testSearchByAutor() throws Exception {
		final String PODLIPNIG = "Stefan Podlipnig";
		final SearchInputMetadataDTO searchMetadata = new SearchInputMetadataDTO(null, null, null, null, PODLIPNIG);
		SearchInputDTO searchQuery = new SearchInputDTO(null, searchMetadata, null, null, null, 0);
		SearchResultsDTO searchResultPage = searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

		org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
		org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
		org.junit.Assert.assertThat(searchResultPage.getSearchResult(), everyItem(
				hasProperty("metadata", hasProperty("creator", hasItemInArray(hasProperty("name", containsString(PODLIPNIG)))))));

	}

	@Test
	public void testProgrammingLanguageSearch() throws Exception {
		final SearchInputMetadataDTO searchMetadata = new SearchInputMetadataDTO("JAVA", null, null, null, null);
		SearchInputDTO searchQuery = new SearchInputDTO(null, searchMetadata, null, null, null, 0);
		SearchResultsDTO searchResultPage = searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

		assertNotNull(searchResultPage.getSearchResult());
		assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
		assertThat(searchResultPage.getSearchResult(), everyItem(
				hasProperty("metadata", hasProperty("programmingLanguage", hasItemInArray(containsString("Java"))))));

	}

	@Test
	public void testCreatorSearch() throws IOException {
		final SearchInputMetadataDTO searchMetadata = new SearchInputMetadataDTO(null, null, null, null, "Podlipnig");
		SearchInputDTO searchQuery = new SearchInputDTO(null, searchMetadata, null, null, null, 0);
		SearchResultsDTO searchResultPage = searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

		org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
		org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
		assertThat(searchResultPage.getSearchResult(), everyItem(hasProperty("metadata",
				hasProperty("creator", hasItemInArray(hasProperty("name", containsString("Podlipnig")))))));

	}

	@Test
	public void testKeywordSearch() throws IOException {
		final SearchInputMetadataDTO searchMetadata = new SearchInputMetadataDTO(null, "latex", null, null, null);
		SearchInputDTO searchQuery = new SearchInputDTO(null, searchMetadata, null, null, null, 0);
		SearchResultsDTO searchResultPage = searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

		org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
		org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
		assertThat(searchResultPage.getSearchResult(),
				everyItem(hasProperty("metadata", hasProperty("keyword", hasItemInArray(containsString("latex"))))));

	}

	@Test
	public void testLicenseSearch() throws IOException {
		final SearchInputMetadataDTO searchMetadata = new SearchInputMetadataDTO(null, null, null, "MIT", null);
		SearchInputDTO searchQuery = new SearchInputDTO(null, searchMetadata, null, null, null, 0);
		SearchResultsDTO searchResultPage = searchService.searchResultPage(searchQuery, 0, SearchInputDTO.PAGE_SIZE);

		org.junit.Assert.assertNotNull(searchResultPage.getSearchResult());
		org.junit.Assert.assertTrue("At least one test hit", searchResultPage.getHitCount() >= 1);
		assertThat(searchResultPage.getSearchResult(),
				everyItem(hasProperty("metadata", hasProperty("license", containsString("MIT")))));

	}
}
